declare class YoniusError {
    constructor(message: string, code?: number)
    isClient(): boolean
    isServer(): boolean
}

declare class OperationalError extends YoniusError {
    constructor(message?: string, code?: number)
}

declare class NotFoundError extends OperationalError {}

declare class NotImplementedError extends OperationalError {}

declare class ValidationError extends OperationalError {}

declare function register(name: string, value: any): void;
declare function unregister(name: string): void;
declare function request(name: string): any;

declare class Model {
    static niw<T = Model>(this: { new(): T }): T
    static find<T = Model>(this: { new(): T }, params?: Record<string, any>): T[]
    static get<T = Model>(this: { new(): T }, params?: Record<string, any>): T

    constructor(options?: { fill?: boolean })
    apply<T = Model>(this: T, model: Record<string, any>): Promise<T>
    save<T = Model>(this: T): Promise<T>
    delete<T = Model>(this: T, options?: {
        preDelete?: boolean,
        postDelete?: boolean,
        beforeCallbacks?: ((self: T, model: Record<string, any>) => void)[],
        afterCallbacks?: ((self: T, model: Record<string, any>) => void)[]
    }): Promise<T>

    validate(): Promise<void>
    _validate(): IterableIterator<Error>
}

declare class ModelStore extends Model {
    static get schema(): Record<string, unknown>
    static get idName(): string
}

declare function verify(condition: boolean, message?: string, code?: number, exception?: any, kwargs?: any): void;

declare function load(names?: Array<string>, path?: string, encoding?: string, force?: boolean, ctx?: any): Promise<void>;
declare function conf(name: string, fallback?: any, cast?: string, ctx?: any): any;

declare function getObject(
    params: Record<string, any>,
    options: {
        alias?: boolean,
        page?: boolean,
        find?: boolean,
        norm?: boolean
    }
): Record<string, any>;

declare function initMongo(
    mongoose: unknown,
    uri: string
): Promise<void>;

declare function destroyMongo(
    mongoose: unknown
): void;

declare function eq<T>(
    valueC: T,
    message?: string
): (value?: T, ctx?: any) => boolean;

declare function gt<T>(
    valueC: T,
    message?: string
): (value?: T, ctx?: any) => boolean;

declare function notEmpty(
    message?: string
): (value?: string | unknown[], ctx?: any) => boolean;

declare function isIn<T>(
    valueC: T[],
    message?: string
): (value?: T, ctx?: any) => boolean;

declare function isSimple(
    message?: string
): (value?: string, ctx?: any) => boolean;

declare function isEmail(
    message?: string
): (value?: string, ctx?: any) => boolean;

declare function isUrl(
    message?: string
): (value?: string, ctx?: any) => boolean;

declare function isRegex(
    regex: string,
    message?: string
): (value?: string, ctx?: any) => boolean;

export { Model, ModelStore, NotFoundError, NotImplementedError, OperationalError, ValidationError, YoniusError, conf, destroyMongo, eq, getObject, gt, initMongo, isEmail, isIn, isRegex, isSimple, isUrl, load, notEmpty, register, request, unregister, verify };
